@@ -48,11 +48,17 @@ pub fn builder(attr: TokenStream, item: TokenStream) -> TokenStream {
 fn process(attr: TokenStream, item: TokenStream) -> Result<TokenStream, TokenStream> {
     let ast = parse::parse(TokenStream::from_iter(vec![attr, item]).into())
         .map_err(|e| e.into_compile_error())?;
-    let model = analyze::analyze(ast.clone()).map_err(|e| e.into_compile_error())?;
-    let ir = lower::lower(model).map_err(|e| e.into_compile_error())?;
+    let constructors = analyze::analyze(ast.clone()).map_err(|e| e.into_compile_error())?;
+    let mut token_streams: Vec<proc_macro::TokenStream> = Vec::new();
+    let original_token_stream = ast.item.to_token_stream().into();
+    token_streams.push(original_token_stream);
 
-    let code_gen = codegen::codegen(ir).map_err(|e| e.into_compile_error())?;
-    let token_stream: TokenStream = ast.item.to_token_stream().into();
+    for constructor_model in constructors {
+        let ir = lower::lower(constructor_model).map_err(|e| e.into_compile_error())?;
 
-    Ok(TokenStream::from_iter(vec![token_stream, code_gen.into()]))
+        let code_gen = codegen::codegen(ir).map_err(|e| e.into_compile_error())?;
+        token_streams.push(code_gen.into());
+    }
+
+    Ok(TokenStream::from_iter(token_streams.into_iter()))
 }
